/* ===========================================================================
               MINIMAL SPANNING TREE OF A UNDIRECTED GRAPH
N is the number of nodes of the graph. N1 is N-1.
distance is the NxN matrix. The element distance[i][j] holds the distance,
which is an integer or a real number, between node i and j of the graph. The
algorithm is correct also for distances which are negative, null, or positive,
and even for non Euclidean distances. We assume that for any node i,
distance[i][i]=0. For real distances, instead of integer distances, write
'double', instead of 'int'. The algorithm is taken from E.W. Dijkstra:
'A Short Introduction to the Art of Programming', EWD 316 August 1971.
=========================================================================== */
#include <iostream.h>
const int N=9; const int N1=8;   // N1 is N-1
typedef int node;                // below:'double D[N][N]' for real distances
void distance_write(int D[N][N], int R, int C)
{for (int i=0; i<R; i=i+1)
     {cout << "\ni=" << i << ":   ";
       for (int j=0; j<C; j=j+1) {cout << D[i][j] <<"  ";};};}

void uv_write(int A[][2], int size)
{for (int i=0; i<size; i=i+1)
          {cout << A[i][0] << " -- " << A[i][1] << "\n";};}

void swap(int &a, int &b) {int temp=a; a=b; b=temp;}
//----------------------------------------------------------------------------
void main()
{cout << "There are N=" << N << " nodes (from 0 to " << N-1 << ").\n";
int uv_arcs [N1][2];
int distance [N][N] = {0,4,9,1,3,9,7,4,2,    // 'double' for real distances
                       0,0,8,5,5,1,5,7,7,
                       0,0,0,6,9,2,5,4,2,
                       0,0,0,0,3,2,5,6,6,
                       0,0,0,0,0,8,9,4,2,
                       0,0,0,0,0,0,6,4,9,
                       0,0,0,0,0,0,0,4,7,
                       0,0,0,0,0,0,0,0,5,
                       0,0,0,0,0,0,0,0,0};
//----------------------------------------------------------------------------
//                    making symmetric the matrix of distances
 for (int i=1; i<N; i=i+1)
{for (int j=0; j<i; j=j+1) {distance[i][j] = distance[j][i];}};
cout << "\nSYMMETRIC MATRIX OF DISTANCES:";
cout << "\nd in row i (=0..N-1) and column j(=0..N-1)";
cout << "\niff d is the distance between node i and node j.\n";
distance_write(distance,N,N); cout << "\n\n";
int i, j, uv_min, p;
int l, uv_minlength;                         // 'double' for real distances
//----------------------------------------------------------------------------
//                    The uv_arcs are the ultraviolet_arcs (see Dijkstra)
//----------------------------------------------------------------------------
int k=0; int h=1;
while (h<=N1) {uv_arcs[h-1][0]=0; uv_arcs[h-1][1]=h; h=h+1;};
while (k<N1)
{//----------------------------------------------------------------------------
 //                   Tracing the outermost loop
 // cout << "Iteration k = " << k << ":\n";
 // cout << "uv_arcs = \n"; uv_write(uv_arcs,N1); cout << "\n";
 //---------------------------------------------------------------------------
 uv_min=k; uv_minlength=distance[uv_arcs[k][0]] [uv_arcs[k][1]];
//----------------------------------------------------------------------------
 i=k+1;
 while (i<N1){l=distance[uv_arcs[i][0]] [uv_arcs[i][1]];
             if (l < uv_minlength) {uv_min=i; uv_minlength=l;};
             i=i+1;};
 p=uv_arcs[uv_min][1];
//----------------------------------------------------------------------------
 if (uv_min!=k) {swap(uv_arcs[k][0],uv_arcs[uv_min][0]);
                 swap(uv_arcs[k][1],uv_arcs[uv_min][1]);};
 i=k+1;
 while (i<N1) {if (distance[uv_arcs[i][0]] [uv_arcs[i][1]]
                   > distance[p] [uv_arcs[i][1]])
                  {uv_arcs[i][0]=p;}; i=i+1;};
 k=k+1;
};
cout << "Arcs of the Minimal Spanning Tree:\n"; uv_write(uv_arcs,N1);
}
/* input:      output:
------------------------------------------------------------------------------
               There are N=9 nodes (from 0 to 8).

               SYMMETRIC MATRIX OF DISTANCES:
               d in row i (=0..N-1) and column j(=0..N-1)
               iff d is the distance between node i and node j.

               i=0:   0  4  9  1  3  9  7  4  2
               i=1:   4  0  8  5  5  1  5  7  7
               i=2:   9  8  0  6  9  2  5  4  2
               i=3:   1  5  6  0  3  2  5  6  6
               i=4:   3  5  9  3  0  8  9  4  2
               i=5:   9  1  2  2  8  0  6  4  9
               i=6:   7  5  5  5  9  6  0  4  7
               i=7:   4  7  4  6  4  4  4  0  5
               i=8:   2  7  2  6  2  9  7  5  0

               Arcs of the Minimal Spanning Tree:
               0 -- 3
               3 -- 5
               5 -- 1
               5 -- 2
               0 -- 8
               8 -- 4
               0 -- 7
               7 -- 6
--------------------------------------------------------------------------- */